/////////////////////////////// Includes //////////////////////////////////////

#include "stdafx.h"
#include "AppSettings.h"
#include <conio.h>


/////////////////////////////// Implementation ////////////////////////////////


#pragma warning(suppress: 26485)
int _tmain(int /*argc*/, TCHAR* /*argv*/[], TCHAR* /*envp*/[]) //NOLINT(modernize-avoid-c-arrays)
{
  //Initialize COM
  const HRESULT hr{CoInitialize(nullptr)};
  if (FAILED(hr))
    return 1;

  CHKLMAppSettings HKLMSettings{_T("Some Company does not exist"), _T("APRODUCT")};
  IAppSettings::String sBufferStd{HKLMSettings.GetProfileString(_T("Section"), _T("Entry"))};


  try
  {
    _tprintf(_T("This program is best run under a debugger as it generates a number of test\n"));
    _tprintf(_T("files on disk and a number of test entries in the registry.\n"));
    _tprintf(_T("Please press a key to continue\n"));
    const int nChar{_getch()};
    UNREFERENCED_PARAMETER(nChar);
    const TCHAR* pszAppName{_T("TestAppSettings")};
    const TCHAR* pszSection{_T("Test Section")};

    {
      //Test out CAppSettingsHKCURegistryStorage
      CHKCUAppSettings registryStorage1{_T("PJ Naughter"), pszAppName, _T("1.0"), true};
      registryStorage1.SetCompanyName(_T("PJ Naughter"));
      IAppSettings::String sCompanyName{registryStorage1.GetCompanyName()};
      registryStorage1.SetProductName(pszAppName);
      IAppSettings::String sProductName{registryStorage1.GetProductName()};
      //registryStorage1.SetProductVersion(_T(""));
      IAppSettings::String sProductVersion{registryStorage1.GetProductVersion()};

      //Test the string persistence functions
      registryStorage1.WriteString(pszSection, _T("Test String"), _T("Test string value"));
      try
      {
        IAppSettings::String sString{registryStorage1.GetString(pszSection, _T("Dummy Test String"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      IAppSettings::String sString{registryStorage1.GetString(pszSection, _T("Test String"))};

      //Test the integer persistence functions
      registryStorage1.WriteInt(pszSection, _T("Test Int"), 33);
      UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
      try
      {
        nValue = registryStorage1.GetInt(pszSection, _T("Dummy Test Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      nValue = registryStorage1.GetInt(pszSection, _T("Test Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)
      nValue = registryStorage1.GetProfileInt(pszSection, _T("Does_not_exist_int"), 42); //NOLINT(clang-analyzer-deadcode.DeadStores)
      sString = registryStorage1.GetProfileString(pszSection, _T("Does_not_exist_str"), _T("Default_str_value"));

      //Test the blob persistence functions
      const char* pData{"1234567890"};
#pragma warning(suppress: 26472 26490)
      registryStorage1.WriteBinary(pszSection, _T("Test Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
      try
      {
        std::vector<BYTE> data(registryStorage1.GetBinary(pszSection, _T("Unknown Test Blob"))); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      std::vector<BYTE> data{registryStorage1.GetBinary(pszSection, _T("Test Blob"))};
      registryStorage1.GetProfileBinary(pszSection, _T("Unknown Test Blob"), data);

      //Test the string array methods
      std::vector<IAppSettings::String> arr;
      arr.emplace_back(_T("First String"));
      arr.emplace_back(_T("Second and Last String"));
      registryStorage1.WriteStringArray(pszSection, _T("Test StringArray"), arr);

      arr = registryStorage1.GetStringArray(pszSection, _T("Test StringArray"));
      try
      {
        arr = registryStorage1.GetStringArray(pszSection, _T("Unknown StringArray")); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      registryStorage1.GetProfileStringArray(pszSection, _T("Unknown StringArray"), arr);

      //Test out the deletion methods
      //registryStorage1.WriteString(pszSection, _T("Test StringArray"), nullptr);
      //registryStorage1.WriteString(pszSection, nullptr, nullptr);

      //Test out GetSections
      std::vector<IAppSettings::String> sections{registryStorage1.GetSections()};
      _tprintf(_T("Found the following sections\n"));
      for (const auto& section : sections)
        _tprintf(_T("%s\n"), section.c_str());

      //Test out the Get and WriteSection methods
      std::vector<IAppSettings::String> sectionEntries;
      sectionEntries = registryStorage1.GetSection(pszSection, true);
      _tprintf(_T("Found the following section entries (Including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());
      sectionEntries = registryStorage1.GetSection(pszSection, false);
      _tprintf(_T("Found the following section entries (Not including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());
      sectionEntries.clear();
      sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
      registryStorage1.WriteSection(pszSection, sectionEntries);
    }

    //Note that since the following section of code writes to HKLM, it will throw exceptions if run from a non-administrator account
    {
      //Test out CAppSettingsHKLMRegistryStorage
      CHKLMAppSettings registryStorage2{_T("PJ Naughter"), pszAppName, _T("1.0"), true};
      registryStorage2.SetCompanyName(_T("PJ Naughter"));
      IAppSettings::String sCompanyName{registryStorage2.GetCompanyName()};
      registryStorage2.SetProductName(pszAppName);
      IAppSettings::String sProductName{registryStorage2.GetProductName()};
      //registryStorage1.SetProductVersion(_T(""));
      IAppSettings::String sProductVersion{registryStorage2.GetProductVersion()};

      //Test the string persistence functions
      IAppSettings::String sSection{_T("Test Section")};
      try
      {
        registryStorage2.WriteString(pszSection, _T("Test String"), _T("Test string value"));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        IAppSettings::String sString{registryStorage2.GetString(pszSection, _T("Dummy Test String"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        IAppSettings::String sString{registryStorage2.GetString(pszSection, _T("Test String"))};
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the integer persistence functions
      try
      {
        registryStorage2.WriteInt(pszSection, _T("Test Int"), 33);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
      try
      {
        nValue = registryStorage2.GetInt(pszSection, _T("Dummy Test Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        nValue = registryStorage2.GetInt(pszSection, _T("Test Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the blob persistence functions
      const char* pData{"1234567890"};
      try
      {
#pragma warning(suppress: 26472 26490)
        registryStorage2.WriteBinary(pszSection, _T("Test Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        std::vector<BYTE> data{registryStorage2.GetBinary(pszSection, _T("Unknown Test Blob"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        std::vector<BYTE> data{registryStorage2.GetBinary(pszSection, _T("Test Blob"))};
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the string array methods
      std::vector<IAppSettings::String> arr;
      arr.emplace_back(_T("First String"));
      arr.emplace_back(_T("Second and Last String"));
      try
      {
        registryStorage2.WriteStringArray(pszSection, _T("Test StringArray"), arr);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        arr = registryStorage2.GetStringArray(pszSection, _T("Test StringArray"));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        arr = registryStorage2.GetStringArray(pszSection, _T("Unknown StringArray")); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }

      //Test out the deletion methods
      //registryStorage2.WriteString(pszSection, _T("Test StringArray"), nullptr);
      //registryStorage2.WriteString(pszSection, nullptr, nullptr);
      
      //Test out GetSections
      std::vector<IAppSettings::String> sections;
      try
      {
        sections = registryStorage2.GetSections();
        _tprintf(_T("Found the following sections\n"));
        for (const auto& section : sections)
          _tprintf(_T("%s\n"), section.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test out the Get and WriteSection methods
      std::vector<IAppSettings::String> sectionEntries;
      try
      {
        sectionEntries = registryStorage2.GetSection(pszSection, true);
        _tprintf(_T("Found the following section entries (Including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      try
      {
        sectionEntries = registryStorage2.GetSection(pszSection, false);
        _tprintf(_T("Found the following section entries (Not including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
      try
      {
        registryStorage2.WriteSection(pszSection, sectionEntries);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
    }

    //Note that since the following section of code writes to HKLM, it will throw exceptions if run from a non-administrator account
    //Also since it is writing to a specific services configuration settings, this sample code uses the service name "PJSERVICE" which 
    //is the name of the service which is provided with my CNTService framework. Note that the values we read and write are just dummy
    //values since that service does not use any configuration parameters
    {
      //Test out CAppSettingsHKLMRegistryStorage
      CServicesAppSettings registryStorage3{_T("PJSERVICE"), false};
      IAppSettings::String sServiceName{registryStorage3.GetServiceName()};
      registryStorage3.SetServiceName(sServiceName);

      //Test the string persistence functions
      IAppSettings::String sSection{_T("Test Section")};
      try
      {
        registryStorage3.WriteString(pszSection, _T("Test String"), _T("Test string value"));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        IAppSettings::String sString{registryStorage3.GetString(pszSection, _T("Dummy Test String"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        IAppSettings::String sString{registryStorage3.GetString(pszSection, _T("Test String"))};
      }
      catch (CAppSettingsException& /*e*/)
      {
      }

      //Test the integer persistence functions
      try
      {
        registryStorage3.WriteInt(pszSection, _T("Test Int"), 33);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
      try
      {
        nValue = registryStorage3.GetInt(pszSection, _T("Dummy Test Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        nValue = registryStorage3.GetInt(pszSection, _T("Test Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the blob persistence functions
      const char* pData{"1234567890"};
      try
      {
#pragma warning(suppress: 26472 26490)
        registryStorage3.WriteBinary(pszSection, _T("Test Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        std::vector<BYTE> data{registryStorage3.GetBinary(pszSection, _T("Unknown Test Blob"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        std::vector<BYTE> data{registryStorage3.GetBinary(pszSection, _T("Test Blob"))};
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the string array methods
      std::vector<IAppSettings::String> arr;
      arr.emplace_back(_T("First String"));
      arr.emplace_back(_T("Second and Last String"));
      try
      {
        registryStorage3.WriteStringArray(pszSection, _T("Test StringArray"), arr);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      try
      {
        arr = registryStorage3.GetStringArray(pszSection, _T("Test StringArray"));
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
      try
      {
        arr = registryStorage3.GetStringArray(pszSection, _T("Unknown StringArray")); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }

      //Test out the deletion methods
      //registryStorage3.WriteString(pszSection, _T("Test StringArray"), nullptr);
      //registryStorage3.WriteString(pszSection, nullptr, nullptr);

      //Test out GetSections
      std::vector<IAppSettings::String> sections;
      try
      {
        sections = registryStorage3.GetSections();
        _tprintf(_T("Found the following sections\n"));
        for (const auto& section : sections)
          _tprintf(_T("%s\n"), section.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test out the Get and WriteSection methods
      std::vector<IAppSettings::String> sectionEntries;
      try
      {
        sectionEntries = registryStorage3.GetSection(pszSection, true);
        _tprintf(_T("Found the following section entries (Including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      try
      {
        sectionEntries = registryStorage3.GetSection(pszSection, false);
        _tprintf(_T("Found the following section entries (Not including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      sectionEntries.clear();
      sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
      try
      {
        registryStorage3.WriteSection(pszSection, sectionEntries);
      }
      catch(CAppSettingsException& /*e*/)
      {
      }
    }

    {
      //Test out CAppSettingsIniFileStorage

      IAppSettings::String sIniFile{CIniAppSettings::GetCurrentProcessIniFileLocation()};
      sIniFile = CIniAppSettings::GetRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T(""), _T("settings.ini"));
      sIniFile = CIniAppSettings::GetRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.ini"));

      sIniFile = CIniAppSettings::GetNonRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T(""), _T("settings.ini"));
      sIniFile = CIniAppSettings::GetNonRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.ini"));

      sIniFile = CIniAppSettings::GetPerComputerDataFileLocation(_T("PJ Naughter"), pszAppName, _T(""), _T("settings.ini"));
      sIniFile = CIniAppSettings::GetPerComputerDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.ini"));

      CIniAppSettings iniStorage1{CIniAppSettings::GetRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.ini"))};


      //Test the string persistence functions
      IAppSettings::String sSection{_T("Test Section")};
      iniStorage1.WriteString(pszSection, _T("Test String"), _T("Test string value"));
      try
      {
        IAppSettings::String sString{iniStorage1.GetString(pszSection, _T("Dummy Test String"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      IAppSettings::String sString{iniStorage1.GetString(pszSection, _T("Test String"))};

      //Test the integer persistence functions
      iniStorage1.WriteInt(pszSection, _T("Test Int"), 33);
      UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
      try
      {
        nValue = iniStorage1.GetInt(pszSection, _T("Dummy Test Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      nValue = iniStorage1.GetInt(pszSection, _T("Test Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)

      //Test the blob persistence functions
      const char* pData{"1234567890"};
#pragma warning(suppress: 26472 26490)
      iniStorage1.WriteBinary(pszSection, _T("Test Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
      try
      {
        std::vector<BYTE> data{iniStorage1.GetBinary(pszSection, _T("Unknown Test Blob"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      try
      {
        std::vector<BYTE> data{iniStorage1.GetBinary(pszSection, _T("Test Blob"))};
      }
      catch(CAppSettingsException& /*e*/)
      {
      }

      //Test the string array methods
      std::vector<IAppSettings::String> arr;
      arr.emplace_back(_T("First String"));
      arr.emplace_back(_T("Second and Last String"));
      iniStorage1.WriteStringArray(pszSection, _T("Test StringArray"), arr);

      arr = iniStorage1.GetStringArray(pszSection, _T("Test StringArray"));
      try
      {
        arr = iniStorage1.GetStringArray(pszSection, _T("Unknown StringArray")); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }

      //Test out the deletion methods
      //iniStorage1.WriteString(pszSection, _T("Test StringArray"), nullptr);
      //iniStorage1.WriteString(pszSection, nullptr, nullptr);

      //Test out GetSections
      std::vector<IAppSettings::String> sections{iniStorage1.GetSections()};
      _tprintf(_T("Found the following sections\n"));
      for (const auto& section : sections)
        _tprintf(_T("%s\n"), section.c_str());

      //Test out the Get and WriteSection methods
      std::vector<IAppSettings::String> sectionEntries{iniStorage1.GetSection(pszSection, true)};
      _tprintf(_T("Found the following section entries (Including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());
      sectionEntries = iniStorage1.GetSection(pszSection, false);
      _tprintf(_T("Found the following section entries (Not including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());
      sectionEntries.clear();
      sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
      iniStorage1.WriteSection(pszSection, sectionEntries);
    }

    {
      //Test out CAppSettingsXMLFileStorage
      CXMLAppSettings xmlStorage1{CIniAppSettings::GetRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.xml")), true, true};

      //Test the string persistence functions
      const TCHAR* pszSection2{_T("Test_Section")};
      xmlStorage1.WriteString(pszSection2, _T("Test_String"), _T("Test string value"));
      try
      {
        IAppSettings::String sString{xmlStorage1.GetString(pszSection2, _T("Dummy_Test_String"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      //xmlStorage1.SetXMLFile(_T("bla"));
      IAppSettings::String sString{xmlStorage1.GetString(pszSection2, _T("Test_String"))};

      //Test the integer persistence functions
      xmlStorage1.WriteInt(pszSection2, _T("Test_Int"), 33);
      UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
      try
      {
        nValue = xmlStorage1.GetInt(pszSection2, _T("Dummy_Test_Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      nValue = xmlStorage1.GetInt(pszSection2, _T("Test_Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)

      //Test the blob persistence functions
      const char* pData{"1234567890"};
#pragma warning(suppress: 26472 26490)
      xmlStorage1.WriteBinary(pszSection2, _T("Test_Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
      try
      {
        std::vector<BYTE> data{xmlStorage1.GetBinary(pszSection2, _T("Unknown_Test_Blob"))}; //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }
      std::vector<BYTE> data{xmlStorage1.GetBinary(pszSection2, _T("Test_Blob"))};

      //Test the string array methods
      std::vector<IAppSettings::String> arr;
      arr.emplace_back(_T("First String"));
      arr.emplace_back(_T("Second and Last String"));
      xmlStorage1.WriteStringArray(pszSection2, _T("Test_StringArray"), arr);

      arr = xmlStorage1.GetStringArray(pszSection2, _T("Test_StringArray"));
      try
      {
        arr = xmlStorage1.GetStringArray(pszSection2, _T("Unknown_StringArray")); //We expect this to throw an exception because it does not exist
      }
#pragma warning(suppress: 26496)
      catch(CAppSettingsException& e)
      {
#pragma warning(suppress: 26472)
        _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
      }

      //Test out the deletion methods
      //xmlStorage1.WriteString(pszSection2, _T("Test_StringArray"), nullptr);
      //xmlStorage1.WriteString(pszSection2, nullptr, nullptr);

      //Test out GetSections
      std::vector<IAppSettings::String> sections{xmlStorage1.GetSections()};
      _tprintf(_T("Found the following sections\n"));
      for (const auto& section : sections)
        _tprintf(_T("%s\n"), section.c_str());

      //Test out the Get and WriteSection methods
      std::vector<IAppSettings::String> sectionEntries{xmlStorage1.GetSection(pszSection2, true)};
      _tprintf(_T("Found the following section entries (Including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());
      sectionEntries = xmlStorage1.GetSection(pszSection2, false);
      _tprintf(_T("Found the following section entries (Not including values)\n"));
      for (const auto& sectionEntry : sectionEntries)
        _tprintf(_T("%s\n"), sectionEntry.c_str());

      sectionEntries.clear();
      sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
      xmlStorage1.WriteSection(pszSection2, sectionEntries);
    }


  #ifdef CAPPSETTINGS_JSON_SUPPORT
    {
      try
      {
        //Test out CJSONAppSettings class
        CJSONAppSettings jsonStorage1{CIniAppSettings::GetRoamingDataFileLocation(_T("PJ Naughter"), pszAppName, _T("1.0"), _T("settings.json")), true};

        //Test the string persistence functions
        const TCHAR* pszSection2{_T("Test_Section")};
        jsonStorage1.WriteString(pszSection2, _T("Test_String"), _T("Test string value"));
        try
        {
          IAppSettings::String sString{jsonStorage1.GetString(pszSection2, _T("Dummy_Test_String"))}; //We expect this to throw an exception because it does not exist
        }
#pragma warning(suppress: 26496)
        catch(CAppSettingsException& e)
        {
#pragma warning(suppress: 26472)
          _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
        }
        //jsonStorage1.SetJSONFile(_T("bla"));
        IAppSettings::String sString{jsonStorage1.GetString(pszSection2, _T("Test_String"))};
        jsonStorage1.WriteString(nullptr, _T("Global_Var"), _T("global_value"), true);
        sString = jsonStorage1.GetString(nullptr, _T("Global_Var"));
        jsonStorage1.WriteInt(_T("Test_Section"), nullptr, 42);

        //Test the integer persistence functions
        jsonStorage1.WriteInt(pszSection2, _T("Test_Int"), 33);
        UINT nValue{0}; //NOLINT(clang-diagnostic-unused-but-set-variable)
        try
        {
          nValue = jsonStorage1.GetInt(pszSection2, _T("Dummy_Test_Int")); //We expect this to throw an exception because it does not exist //NOLINT(clang-analyzer-deadcode.DeadStores)
        }
#pragma warning(suppress: 26496)
        catch(CAppSettingsException& e)
        {
#pragma warning(suppress: 26472)
          _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
        }
        nValue = jsonStorage1.GetInt(pszSection2, _T("Test_Int")); //NOLINT(clang-analyzer-deadcode.DeadStores)

        //Test the blob persistence functions
        const char* pData{"1234567890"};
#pragma warning(suppress: 26472 26490)
        jsonStorage1.WriteBinary(pszSection2, _T("Test_Blob"), reinterpret_cast<const BYTE*>(pData), static_cast<DWORD>(strlen(pData)));
        try
        {
          std::vector<BYTE> data{jsonStorage1.GetBinary(pszSection2, _T("Unknown_Test_Blob"))}; //We expect this to throw an exception because it does not exist
        }
#pragma warning(suppress: 26496)
        catch(CAppSettingsException& e)
        {
#pragma warning(suppress: 26472)
          _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
        }
        std::vector<BYTE> data{jsonStorage1.GetBinary(pszSection2, _T("Test_Blob"))};

        //Test the string array methods
        std::vector<IAppSettings::String> arr;
        arr.emplace_back(_T("First String"));
        arr.emplace_back(_T("Second and Last String"));
        jsonStorage1.WriteStringArray(pszSection2, _T("Test_StringArray"), arr);

        arr = jsonStorage1.GetStringArray(pszSection2, _T("Test_StringArray"));
        try
        {
          arr = jsonStorage1.GetStringArray(pszSection2, _T("Unknown_StringArray")); //We expect this to throw an exception because it does not exist
        }
#pragma warning(suppress: 26496)
        catch(CAppSettingsException& e)
        {
#pragma warning(suppress: 26472)
          _tprintf(_T("An expected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
        }

        //Test out the deletion methods
        //jsonStorage1.WriteString(pszSection2, _T("Test_StringArray"), nullptr);
        //jsonStorage1.WriteString(pszSection2, nullptr, nullptr);

        //Test out GetSections
        std::vector<IAppSettings::String> sections{jsonStorage1.GetSections()};
        _tprintf(_T("Found the following sections\n"));
        for (const auto& section : sections)
          _tprintf(_T("%s\n"), section.c_str());

        //Test out the Get and WriteSection methods
        std::vector<IAppSettings::String> sectionEntries{jsonStorage1.GetSection(pszSection2, true)};
        _tprintf(_T("Found the following section entries (Including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());
        sectionEntries = jsonStorage1.GetSection(pszSection2, false);
        _tprintf(_T("Found the following section entries (Not including values)\n"));
        for (const auto& sectionEntry : sectionEntries)
          _tprintf(_T("%s\n"), sectionEntry.c_str());

        sectionEntries.clear();
        sectionEntries.emplace_back(_T("OnlyValue=SomeValue"));
        jsonStorage1.WriteSection(pszSection2, sectionEntries);
      }
#pragma warning(suppress: 26496)
      catch(JSONPP::CParseException& e)
      {
        _tprintf(_T("An unexpected JSONPP::CParseException exception has occurred, Reason:%d\n"), e.m_Reason);
      }
    }
  #endif //#ifdef CAPPSETTINGS_JSON_SUPPORT
  }
#pragma warning(suppress: 26496)
  catch(CAppSettingsException& e)
  {
#pragma warning(suppress: 26472)
    _tprintf(_T("An unexpected CAppSettingsException exception has occurred, Error:%08X\n"), static_cast<unsigned int>(e.m_hr));
  }

  CoUninitialize();
  return 0;
}
